@@ -197,6 +197,10 @@ module Async
 				@io.connected?
 			end
 			
+			def readable?
+				@io.readable?
+			end
+			
 			def closed?
 				@io.closed?
 			end
@@ -246,6 +250,21 @@ module Async
 			
 			private
 			
+			def sysread(size, buffer)
+				while true
+					result = @io.read_nonblock(size, buffer, exception: false)
+					
+					case result
+					when :wait_readable
+						@io.wait_readable
+					when :wait_writable
+						@io.wait_writable
+					else
+						return result
+					end
+				end
+			end
+			
 			# Fills the buffer from the underlying stream.
 			def fill_read_buffer(size = @block_size)
 				# We impose a limit because the underlying `read` system call can fail if we request too much data in one go.
@@ -257,12 +276,12 @@ module Async
 				flush
 				
 				if @read_buffer.empty?
-					if @io.read_nonblock(size, @read_buffer, exception: false)
+					if sysread(size, @read_buffer)
 						# Console.logger.debug(self, name: "read") {@read_buffer.inspect}
 						return true
 					end
 				else
-					if chunk = @io.read_nonblock(size, @input_buffer, exception: false)
+					if chunk = sysread(size, @input_buffer)
 						@read_buffer << chunk
 						# Console.logger.debug(self, name: "read") {@read_buffer.inspect}
 						
